/*+
    Name:	vgaregs.h
    Date:	30-Aug-1988
    Author:	Kent J. Quirk
    Abstract:	This file declares all the structures and routines used
		to access VGA registers.
-*/

#pragma intrinsic (inp, outp)	/* use fast code for port read/write */
#pragma loop_opt(off)		/* don't want memory read loops optimized */

typedef struct {
    BYTE m_out,
	 f_ctrl,
	 istat0,
	 istat1;
} GENERAL_REGS;

#define VRETRACE    0x08
#define DISP_EN_NOT 0x01

typedef struct {
    BYTE clk_mode,
	 map_mask,
	 char_map,
	 mem_mode;
} SEQUENCER_REGS;

#define SQ_CLK_MODE 1
#define SQ_MAP_MASK 2
#define SQ_CHAR_MAP 3
#define SQ_MEM_MODE 4

typedef struct {
    BYTE red, blu, grn;
} DAC_COLOR;

typedef struct {
    DAC_COLOR col[256];
} DAC_REGS;

#define DAC_PEL_ADDR_W	0x3C8
#define DAC_PEL_ADDR_R	0x3C7
#define DAC_STATE	0x3C7
#define DAC_PEL_DATA	0x3C9
#define DAC_PEL_MASK	0x3C6

typedef struct {
    BYTE h_total,
	 h_disp_end,
	 h_blank_start,
	 h_blank_end,
	 h_retrace_start,
	 h_retrace_end,
	 v_total,
	 overflow,
	 preset_row_scan,
	 max_scan_line,
	 cur_start,
	 cur_end,
	 start_addr_hi,
	 start_addr_lo,
	 curs_loc_hi,
	 curs_loc_lo,
	 v_retrace_start,
	 v_retrace_end,
	 v_display_end,
	 offset,
	 underline_loc,
	 v_blank_start,
	 v_blank_end,
	 mode_control,
	 line_compare;
} CRTC_REGS;

#define H_TOTAL 	0x0
#define H_DISP_END	0x1
#define H_BLANK_START	0x2
#define H_BLANK_END	0x3
#define H_RETRACE_START 0x4
#define H_RETRACE_END	0x5
#define V_TOTAL 	0x6
#define OVERFLOW	0x7
#define PRESET_ROW_SCAN 0x8
#define MAX_SCAN_LINE	0x9
#define CUR_START	0xA
#define CUR_END 	0xB
#define START_ADDR_HI	0xC
#define START_ADDR_LO	0xD
#define CURS_LOC_HI	0xE
#define CURS_LOC_LO	0xF
#define V_RETRACE_START 0x10
#define V_RETRACE_END	0x11
#define V_DISPLAY_END	0x12
#define OFFSET		0x13
#define UNDERLINE_LOC	0x14
#define V_BLANK_START	0x15
#define V_BLANK_END	0x16
#define CRTC_MODE_CTRL	0x17
#define LINE_COMPARE	0x18

#define VRET_ENABLE	0x20
#define VRET_CLR	0x10
#define VRET_INT	0x0A

typedef struct {
    BYTE set_reset,
	 enable_set_reset,
	 color_compare,
	 data_rotate,
	 read_map_select,
	 mode,
	 misc,
	 color_dont_care,
	 bit_mask;
} GRAPHICS_REGS;

#define SET_RESET	 0
#define ENABLE_SET_RESET 1
#define COLOR_COMPARE	 2 
#define DATA_ROTATE	 3
#define READ_MAP_SELECT  4
#define MODE		 5
#define MISC		 6
#define COLOR_DONT_CARE  7
#define BIT_MASK	 8

#define WRITE_MODE_MASK    0x03
#define READ_MODE_MASK	   0x08
#define ODD_EVEN_MASK	   0x10
#define SHIFT_REG_CTRL_0   0x20
#define SHIFT_REG_CTRL_1   0x40

typedef struct {
    BYTE palette,
	 mode_control,
	 overscan,
	 color_plane_enable,
	 h_pel_panning,
	 color_select;
} ATTRIB_REGS;

#define PALETTE_BASE	   0
#define ATTR_MODE_CTRL	   0x10 | 0x20		/* 20 = proc. data select */
#define OVERSCAN	   0x11 | 0x20
#define COLOR_PLANE_ENABLE 0x12 | 0x20
#define H_PEL_PANNING	   0x13 | 0x20
#define COLOR_SELECT	   0x14 | 0x20
			    
typedef struct {
    GENERAL_REGS    gen;
    SEQUENCER_REGS  seq;
    CRTC_REGS	    crtc;
    GRAPHICS_REGS   graph;
    ATTRIB_REGS     attr;
} ALL_REGS;

#define MONOVID 0xB0
#define COLORVID 0xD0

extern int vidmode;

/* General registers */
#define MISC_OUT_W	0x3C2
#define MISC_OUT_R	0x3CC
#define FEATURE_CTRL_W	(0x30A + vidmode)
#define FEATURE_CTRL_R	0x3CA
#define INPUT_STATUS_0	0x3C2
#define INPUT_STATUS_1	(0x30A + vidmode)

/* Attribute registers */
#define ATTR_INDEX	0x3C0
#define ATTR_OTHER_W	0x3C0
#define ATTR_OTHER_R	0x3C1
#define ATTR_INIT	INPUT_STATUS_1

/* CRT Controller */
#define CRT_INDEX	(0x304 + vidmode)
#define CRT_OTHER	(0x305 + vidmode)

/* Sequencer */
#define SEQ_INDEX	0x3C4
#define SEQ_OTHER	0x3C5

/* Graphics */
#define GRAPH_INDEX	0x3CE
#define GRAPH_OTHER	0x3CF

/* DAC Interface */
#define DAC_PIXEL_MASK	0x3C6
#define DAC_READ_DATA	0x3C7
#define DAC_STATUS	0x3C7
#define DAC_WRITE_DATA	0x3C8
#define DAC_OUTPUT_DATA 0x3C9

#define read_general_reg(r) (BYTE)inp(r)
#define write_general_reg(r, v) outp(r, v)

void write_indexed_reg(WORD base, WORD write, BYTE index, BYTE value);
BYTE read_indexed_reg(WORD base, WORD read, BYTE index);
BYTE read_dac_reg(WORD r);
void write_dac_reg(WORD r, BYTE v);
BYTE read_attrib_reg(WORD r);
void write_attrib_reg(WORD r, BYTE v);

void read_all_general_regs(GENERAL_REGS *r);
void read_all_attrib_regs(ATTRIB_REGS *r);
void read_all_sequencer_regs(SEQUENCER_REGS *r);
void read_all_graph_regs(GRAPHICS_REGS *r);
void read_all_crtc_regs(CRTC_REGS *r);

void wait_for_vertical(int fast);

#define read_sequencer_reg(r) read_indexed_reg(SEQ_INDEX, SEQ_OTHER, r)
#define write_sequencer_reg(r, v) write_indexed_reg(SEQ_INDEX, SEQ_OTHER, r, v)
#define read_crtc_reg(r) read_indexed_reg(CRT_INDEX, CRT_OTHER, r)
#define write_crtc_reg(r, v) write_indexed_reg(CRT_INDEX, CRT_OTHER, r, v)
#define read_graph_reg(r) read_indexed_reg(GRAPH_INDEX, GRAPH_OTHER, r)        
#define write_graph_reg(r, v) write_indexed_reg(GRAPH_INDEX, GRAPH_OTHER, r, v)

#define CMP_GENERAL	0
#define CMP_DAC 	1
#define CMP_SEQUENCER	2
#define CMP_CRTC	3
#define CMP_ATTRIB	4
#define CMP_STATE	5
